#!/usr/bin/python3

from PyQt6.QtCore import Qt
from PyQt6.QtWidgets import QDockWidget, QFileDialog
from .ui.main_window import MainWindow


class GUI:
    def __init__(self, app):
        self.app = app

        self.dropped = []

        self.window = MainWindow(app, self)
        self.settings = self.window.settings
        self.track_control = self.window.track_control
        self.process_dock = self.window.process_dock
        self.track_info = self.window.track_info

        self.window.addDockWidget(Qt.DockWidgetArea.LeftDockWidgetArea, self.app.library.main_library_dock)

        self.app.library.main_library_dock.setFeatures(
            QDockWidget.DockWidgetFeature.DockWidgetMovable |
            QDockWidget.DockWidgetFeature.DockWidgetFloatable
        )

        if self.app.settings.window_maximized:
            self.window.showMaximized()

        elif self.app.settings.window_size is not None:
            self.window.resize(*self.app.settings.window_size)

        self.audio_file_selector = QFileDialog(self.window, "Select Audio Files")
        self.audio_file_selector.setFileMode(QFileDialog.FileMode.ExistingFiles)
        self.audio_file_selector.setNameFilters(["Audio Files (*.flac *.wav *.mp3 *.ogg *.opus)", "Any (*)"])
        self.audio_file_selector.setViewMode(QFileDialog.ViewMode.List)

        self.playlist_file_selector = QFileDialog(self.window, "Select Playlist")
        self.playlist_file_selector.setFileMode(QFileDialog.FileMode.ExistingFile)
        self.playlist_file_selector.setNameFilters(["Playlists (*.wbz.m3u *.m3u)", "Any (*)"])
        self.playlist_file_selector.setViewMode(QFileDialog.ViewMode.List)

        self.connect()

        self.window.show()

        self.settings.update_all()

    def connect(self):
        self.window.closeEvent = self.on_exit

    def on_exit(self, event):
        self.app.library.on_exit(event)

        self.app.settings.window_size = (self.window.width(), self.window.height())
        self.app.settings.window_maximized = self.window.isMaximized()

        self.app.settings.save(self.app.utils.settings_location)

    def on_settings_change(self, key, value):
        self.settings.update_settings(key, value)

    def on_track_change(self, previous_track, track):
        self.track_control.on_track_change(previous_track, track)

        for dock_id in self.app.player.current_playlist.views:
            view = self.app.player.current_playlist.views[dock_id]
            view.on_track_change(previous_track, track)

    def on_background_job_start(self, job_name: str, description: str, steps: int=0, getter: any=None):
        self.process_dock.job_started_signal.emit(job_name, description, steps, getter)

    def on_background_job_stop(self, job_name: str):
        self.process_dock.job_finished_signal.emit(job_name)

    def on_playstate_update(self):
        self.track_control.on_playstate_update()
        self.track_info.update_info()

    def select_audio_files(self):
        if self.audio_file_selector.exec():
            return self.audio_file_selector.selectedFiles()

    def select_playlist_file(self):
        if self.playlist_file_selector.exec():
            return self.playlist_file_selector.selectedFiles()[0]

    def open_tracks(self):
        files = self.select_audio_files()

        if files is not None and not files == []:
            self.app.library.open_tracks(files)

    def import_tracks(self):
        self.open_tracks()  # placeholder

    def import_playlist(self):
        playlist_path = self.select_playlist_file()

        if playlist_path is not None and not playlist_path == "":
            self.app.library.import_playlist(playlist_path)

